/*

-Procedure track_target_onerev ( return a time series of vectors pointing 
                                 from a moving observer to a moving target,
                                 over one orbital revolution of the target 
                                 around its central body.  )

     Jeff Bytof, last update 1-May-2001

*/

#include "SpiceUsr.h"
#include "SpiceZmc.h"
#include "SpiceZfc.h"

void track_target_onerev ( SpiceInt         cenid,
                           SpiceInt         trgid,
                           SpiceInt         obsid, 
                           SpiceDouble      etstart,
                           SpiceInt         npts,
                           SpiceDouble      etlist[],
                           SpiceDouble      poslist[][3],
                           SpiceDouble      orbpars[10]   )

{


/*

-Brief_I/O

    Variable    I/O      Description
    --------    ---      -----------
    cenid        I       NAIF ID code of central body
    orbid        I       NAIF ID code of target body, which orbits
                         around cenid.
    obsid        I       NAIF ID code of observer. 
    etstart      I       Ephemeris time of first point.
    npts         O       Number of generated positions requested. 
    etlist       O       Array of times.
    poslist      O       List of relative positions of the orbiting
                         body relative to the observing point, 
                         expressed in the J2000 reference frame.
    orbpars      O       List of osculating orbit parameters of the 
                         target body, its orbital period around the 
                         central body and the gravitational parameter, mu, 
                         of the central body. 

*/
   
/*
   Local variables.
*/

   SpiceDouble   eccen;
   SpiceDouble   elts     [8];
   SpiceDouble   etstep;
   SpiceDouble   et;
   SpiceDouble   lt; 
   SpiceDouble   mu; 
   SpiceDouble   period;
   SpiceDouble   radperi; 
   SpiceDouble   sma; 
   SpiceDouble   state    [6]; 

   SpiceInt      dim;
   SpiceInt      i;
 

/*
    Obtain the gravitation parameter of the central
    body, mu (km^3/sec^2)
*/

    bodvar_c ( cenid, "GM", &dim, &mu );

    if( dim != 1 ) 
    {
       printf("track_target_onerev: mu not found for cenid = %d\n", cenid);
       exit(1);
    } 
      

/*
    Get the state of the satellite at the start epoch. 
*/

    spkez_c ( trgid, 
              etstart,
              "J2000",
              "LT+S", 
              cenid,
              state,
              &lt    ); 


/*
    From the state, obtain the conic elements at the start epoch.
*/
  
    oscelt_c ( state,
               etstart,
               mu,
               elts );

    radperi = elts[0];
    eccen   = elts[1];

    if( eccen >= 1.0 ) 
    {
       printf("track_target_onerev: orbit not elliptical, 
          eccen = %f\n", eccen);
       exit(1);
    }

/*
    Get semi-major axis.
*/ 

    sma = radperi / (1.0-eccen);


/*
    Get orbital period
*/

    period = twopi_c() * pow( sma, 1.5 ) / sqrt(mu);

/*
    Track the target for one orbital revolution. 
*/

    etstep = period / (double) npts; 

    track_target ( trgid,
                   obsid,
                   etstart, 
                   etstep,
                   npts,
                   etlist, 
                   poslist );   

/*
    Store orbital parameters for output
*/

    for( i=0; i<8; i++ ) 
    { 
       orbpars[i] = elts[i];
    }

    orbpars[8]  = sma;
    orbpars[9]  = period;

    return;

}
 

